@@ -47,25 +47,24 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* VOID:STRING,STRING,BOOLEAN (yelp-marshal.list:1) */
+/* VOID:POINTER,BOOLEAN (yelp-marshal.list:1) */
 void
-yelp_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
-                                          GValue       *return_value,
-                                          guint         n_param_values,
-                                          const GValue *param_values,
-                                          gpointer      invocation_hint,
-                                          gpointer      marshal_data)
+yelp_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
+                                    GValue       *return_value,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint,
+                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN) (gpointer     data1,
-                                                            gpointer     arg_1,
-                                                            gpointer     arg_2,
-                                                            gboolean     arg_3,
-                                                            gpointer     data2);
-  register GMarshalFunc_VOID__STRING_STRING_BOOLEAN callback;
+  typedef void (*GMarshalFunc_VOID__POINTER_BOOLEAN) (gpointer     data1,
+                                                      gpointer     arg_1,
+                                                      gboolean     arg_2,
+                                                      gpointer     data2);
+  register GMarshalFunc_VOID__POINTER_BOOLEAN callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 4);
+  g_return_if_fail (n_param_values == 3);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -77,12 +76,11 @@ yelp_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_STRING_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__POINTER_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_string (param_values + 1),
-            g_marshal_value_peek_string (param_values + 2),
-            g_marshal_value_peek_boolean (param_values + 3),
+            g_marshal_value_peek_pointer (param_values + 1),
+            g_marshal_value_peek_boolean (param_values + 2),
             data2);
 }
 
