@@ -37,8 +37,7 @@
 #include "yelp-util.h"
 #include "yelp-scrollkeeper.h"
 
-#define d(x) x
-#undef DEBUG_OUTPUT
+#define d(x)
 
 static void   yvh_init                      (YelpViewTOC      *html);
 static void   yvh_class_init                (YelpViewTOCClass *klass);
@@ -51,14 +50,14 @@ static void   yelp_view_toc_man_2           (YelpViewTOC      *view,
 static void   yelp_view_read_important_docs (YelpViewTOC      *view);
 
 enum {
-	URL_SELECTED,
+	URI_SELECTED,
 	LAST_SIGNAL
 };
 
 static gint signals[LAST_SIGNAL] = { 0 };
 
 typedef struct {
-	char *title;
+	char  *title;
 	GList *seriesids;
 } YelpImportantDocsSection;
 
@@ -73,6 +72,95 @@ struct _YelpViewTOCPriv {
 	int           buffer_pos;
 };
 
+/* HTML generation stuff */
+#define BG_COLOR "#ffffff"
+#define BLOCK_BG_COLOR "#c1c1c1"
+#define HELP_IMAGE "file:///gnome/head/INSTALL/share/pixmaps/gnome-help.png"
+
+#define PAGE_HEADER "<html>\n \
+  <head>\n \
+    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n \
+    <title>\n \
+      %s\n \
+    </title> \n \
+    <style type=\"text/css\">\n \
+      A:link { color: #00008b; font-size: 10 }\n \
+      A:visited { color: #00008b; font-size: 10}\n \
+      A:active { color: #00008b; font-size: 10}\n \
+      h1 { font-size: 25 }\n \
+      h2 { font-size: 15 }\n \
+      BODY { color: #000000; font-size: 10 }\n \
+    </style>\n \
+  </head>\n \
+  <body bgcolor=\"" BG_COLOR "\">"
+
+#define PAGE_START "<table cellpadding=\"0\" \n\
+			      border=\"0\" \n \
+ 			      align=\"center\" \n \
+			      cellspacing=\"10\" \n\
+			      width=\"100%\"> \n\
+			      <tr align=\"left\" \n \
+			          valign=\"top\" \n \
+			          bgcolor=\"" BG_COLOR "\"> \n\
+			      <td height=\"69\" \n\
+			          align=\"center\" \n\
+			          valign=\"center\" \n\
+			          bgcolor=\"" BLOCK_BG_COLOR "\" \n\
+			          colspan=\"2\"> \n\
+			        <h1> \n\
+			          %s \n\
+			        </h1> \n\
+			      </td> \n\
+			      </tr> \n\
+			      <tr>\n"
+
+#define PAGE_END "</tr></table>\n"
+
+#define COLUMN_LEFT_START "<td valign=\"top\" width=\"50%\"> \n\
+            <table cellpadding=\"0\" \n\
+                   border=\"0\"\n\
+		   align=\"left\" \n\
+		   cellspacing=\"0\" \n\
+		   width=\"100%\"> \n\
+              <tr> \n\
+	        <td>\n"
+#define COLUMN_RIGHT_START "<td valign=\"top\"> \n\
+			      <table cellpadding=\"0\" \n\
+			             border=\"0\" \n\
+			             align=\"right\" \n\
+			             cellspacing=\"0\" \n\
+			             width=\"100%\"><tr><td>\n"
+
+#define COLUMN_END " \n\
+		</td> \n\
+              </tr> \n\
+            </table> \n\
+          </td>\n"
+
+#define TOC_BLOCK_START " \
+		  <table cellpadding=\"5\" \n\
+		         border=\"0\" \n\
+			 align=\"left\" \n\
+			 cellspacing=\"0\" \n\
+			 width=\"100%\" \n\
+			 bgcolor=\"" BLOCK_BG_COLOR "\"> \n\
+		    <tr> \n\
+		      <td>\n"
+
+#define TOC_BLOCK_END "\n \
+		      </td> \n\
+		    </tr> \n\
+		  </table>\n "
+
+#define TOC_BLOCK_SEPARATOR " \n\
+                </td>\n \
+	      </tr> \n\
+              <tr> \n\
+                <td>&nbsp;</td> \n\
+              </tr> \n\
+	      <tr>\n\
+	        <td>\n"
+
 GType
 yelp_view_toc_get_type (void)
 {
@@ -122,27 +210,32 @@ yvh_init (YelpViewTOC *view)
 static void
 yvh_class_init (YelpViewTOCClass *klass)
 {
-	signals[URL_SELECTED] = 
-		g_signal_new ("url_selected",
+	signals[URI_SELECTED] = 
+		g_signal_new ("uri_selected",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (YelpViewTOCClass,
-					       url_selected),
+					       uri_selected),
 			      NULL, NULL,
-			      yelp_marshal_VOID__STRING_STRING_BOOLEAN,
+			      yelp_marshal_VOID__POINTER_BOOLEAN,
 			      G_TYPE_NONE,
-			      3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+			      2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
 }
 
 static void
 yvh_link_clicked_cb (HtmlDocument *doc, const gchar *url, YelpViewTOC *view)
 {
+	YelpURI *uri;
+	
 	g_return_if_fail (HTML_IS_DOCUMENT (doc));
 	g_return_if_fail (url != NULL);
 	g_return_if_fail (YELP_IS_VIEW_TOC (view));
-	
-	g_signal_emit (view, signals[URL_SELECTED], 0,
-		       url, NULL, FALSE);
+
+	d(g_print ("Link clicked: %s\n", url));
+
+	uri = yelp_uri_new (url);
+	g_signal_emit (view, signals[URI_SELECTED], 0, uri, FALSE);
+	yelp_uri_unref (uri);
 }
 
 static void
@@ -176,9 +269,7 @@ yelp_view_toc_write (YelpViewTOC *view, char *data, int len)
 		len = strlen (data);
 	}
 
-#ifdef DEBUG_OUTPUT
-	g_print ("%.*s", len,data);
-#endif
+	d(g_print ("%.*s", len,data));
 
 	while (len > 0) {
 		chunk_size = MIN (BUFFER_SIZE - priv->buffer_pos, len);
@@ -189,55 +280,29 @@ yelp_view_toc_write (YelpViewTOC *view, char *data, int len)
 		data += chunk_size;
 
 		if (priv->buffer_pos == BUFFER_SIZE) {
-			html_document_write_stream (priv->doc, priv->buffer, BUFFER_SIZE);
+			html_document_write_stream (priv->doc, 
+						    priv->buffer, 
+						    BUFFER_SIZE);
 			priv->buffer_pos = 0;
 		}
-		
 	}
 }
 
 static void
 yelp_view_toc_printf (YelpViewTOC *view, char *format, ...)
 {
-  va_list args;
-  gchar *string;
-  
-  g_return_if_fail (format != NULL);
-  
-  va_start (args, format);
-  string = g_strdup_vprintf (format, args);
-  va_end (args);
-
-  yelp_view_toc_write (view, string, -1);
-  
-  g_free (string);
-}
-
-static void
-yelp_view_toc_write_header (YelpViewTOC *view, char *title)
-{
-	char *header="\n"
-"<html>\n"
-"  <head>\n"
-"    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
-"    <title>\n"
-"      %s\n"
-"    </title> \n"
-"    <style type=\"text/css\">\n"
-"      A:link { color: #00008b }\n"
-"      A:visited { color: #00008b }\n"
-"      A:active { color: #00008b }\n"
-"      BODY { color: #00008b }\n"
-"    </style>\n"
-"  </head>\n"
-"  <body bgcolor=\"#ffffff\">";
-	char *s;
-
-	s = g_strdup_printf (header, title);
-
-	yelp_view_toc_write (view, s, -1);
-
-	g_free (s);
+	va_list  args;
+	gchar   *string;
+	
+	g_return_if_fail (format != NULL);
+	
+	va_start (args, format);
+	string = g_strdup_vprintf (format, args);
+	va_end (args);
+	
+	yelp_view_toc_write (view, string, -1);
+	
+	g_free (string);
 }
 
 static void
@@ -256,27 +321,19 @@ yelp_view_toc_start (YelpViewTOC *view)
 	YelpViewTOCPriv *priv;
 	GNode           *node;
 	YelpSection     *section;
-	gchar           *table_start = "\n"
-"    <center>\n"
-"      <table cellspacing=\"20\" width=\"100%\">\n"
-"        <tr>\n";
-	gchar           *table_end = "\n"
-"      </tr>\n"
-"    </table>\n"
-"   </center>\n";
 	char            *seriesid;
 	GNode           *root;
 	char            *path;
 	GList           *sections, *seriesids;
-	gchar           *other_docs_string = _("Other Documents");
-	gchar           *man_string = _("Manual pages");
-	gchar           *info_string = _("Info pages");
-	gchar           *installed_string = _("Installed documents");
+	gchar           *page_title = _("Help Contents");
+	gchar           *section_gnome = _("GNOME - Desktop");
+	gchar           *section_additional = _("Additional documents");
+	gchar           *man_string = _("Manual Pages");
+	gchar           *info_string = _("Info Pages");
 	YelpImportantDocsSection *important_section;
 	gboolean         important_doc_installed = FALSE;
-	gboolean         man_installed = FALSE;
-	gboolean         info_installed = FALSE;
-      
+	gboolean         left_column_started = FALSE;
+	
 	priv = view->priv;
 
 	if (!g_node_first_child (priv->doc_tree)) {
@@ -285,14 +342,14 @@ yelp_view_toc_start (YelpViewTOC *view)
 	
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, _("Start page"));
+	yelp_view_toc_printf (view, PAGE_HEADER, page_title);
 	
-	yelp_view_toc_write (view, table_start, -1);
-
 	yelp_view_toc_printf (view, 
-			      "<td valign=\"top\">\n");
-
+			      PAGE_START,
+			      page_title);
+	
 	sections = priv->important_sections;
+	
 	while (sections != NULL) {
 		important_section = sections->data;
 
@@ -312,9 +369,16 @@ yelp_view_toc_start (YelpViewTOC *view)
 		}
 		
  		if (important_doc_installed) {
+			if (!left_column_started) {
+				yelp_view_toc_printf (view, 
+						      COLUMN_LEFT_START);
+				left_column_started = TRUE;
+			}
+			
 			yelp_view_toc_printf (view, 
-					      "<h2>%s</h2>\n"
-					      "<ul>\n", 
+					      TOC_BLOCK_SEPARATOR
+					      TOC_BLOCK_START 
+					      "<h2>%s</h2>",
 					      important_section->title);
 		}
 		
@@ -328,31 +392,45 @@ yelp_view_toc_start (YelpViewTOC *view)
 			if (node) {
 				section = node->data;
 				yelp_view_toc_printf (view, 
-						      "<li><a href=\"%s\">%s</a>\n",
-						      section->uri, section->name);
+						      "<a href=\"%s\">%s</a>\n",
+						      yelp_uri_to_string (section->uri), section->name);
 			}
+
 			seriesids = seriesids->next;
 		}
 
-		yelp_view_toc_printf (view, "</ul>\n");
+		if (important_doc_installed) {
+			yelp_view_toc_printf (view, 
+					      TOC_BLOCK_END);
+			if (sections->next) {
+				yelp_view_toc_printf (view, 
+						      TOC_BLOCK_SEPARATOR);
+			}
+		}
 		
 		sections = sections->next;
+		important_doc_installed = FALSE;
+	}
+
+	if (left_column_started) {
+		yelp_view_toc_printf (view, COLUMN_END);
 	}
 	
-			      
-	yelp_view_toc_write (view, 
-			     "</ul>\n"
-			     "</td>\n", -1);
-
-
-
 	yelp_view_toc_printf (view, 
-			     "<td valign=\"top\">\n"
-			     "<h2>%s</h2>\n", installed_string);
+			      COLUMN_RIGHT_START);
 
 	root = yelp_util_find_toplevel (priv->doc_tree, "scrollkeeper");
 	node = g_node_first_child (root);
 
+	yelp_view_toc_printf (view,
+			      TOC_BLOCK_SEPARATOR
+			      TOC_BLOCK_START
+			      "<h2>%s</h2>\n",
+			      section_gnome);
+			      
+	root = yelp_util_find_node_from_name (priv->doc_tree, "GNOME");
+	node = g_node_first_child (root);
+
 	while (node) {
 		section = YELP_SECTION (node->data);
 		path = yelp_util_node_to_string_path (node);
@@ -364,37 +442,48 @@ yelp_view_toc_start (YelpViewTOC *view)
 		node = g_node_next_sibling (node);
 	}
 
-	if (yelp_util_find_toplevel (priv->doc_tree, "man")) {
-		man_installed = TRUE;
-	}
-	
-	if (yelp_util_find_toplevel (priv->doc_tree, "info")) {
-		info_installed = TRUE;
-	}
-	
-	
-	if (man_installed || info_installed) {
-		yelp_view_toc_printf (view, "<h2>%s</h2>\n", 
-				      other_docs_string);
-		
-		if (man_installed) {
+	yelp_view_toc_printf (view,
+			      TOC_BLOCK_END);
+
+	yelp_view_toc_printf (view,
+			      TOC_BLOCK_SEPARATOR
+			      TOC_BLOCK_START
+			      "<h2>%s</h2>\n",
+			      section_additional);
+
+	root = yelp_util_find_toplevel (priv->doc_tree, "scrollkeeper");
+	node = g_node_first_child (root);
+
+	while (node) {
+		section = YELP_SECTION (node->data);
+		if (strcmp (section->name, "GNOME")) {
+			path = yelp_util_node_to_string_path (node);
 			yelp_view_toc_printf (view, 
-					      "<a href=\"toc:man\">%s</a><br>\n",
-					      man_string);
-		}
-		
-		if (info_installed) {
-			yelp_view_toc_printf (view,
-					      "<a href=\"toc:info\">%s</a><br>\n",
-					      info_string);
+					      "<a href=\"toc:scrollkeeper/%s\">%s</a><br>\n", 
+					      path, section->name);
+			g_free (path);
+			
 		}
+		node = g_node_next_sibling (node);
 	}
+
+	if (yelp_util_find_toplevel (priv->doc_tree, "man")) {
+		yelp_view_toc_printf (view,
+				      "<a href=\"toc:man\">%s</a><br>\n",
+				      man_string);
+	}
+		
+	if (yelp_util_find_toplevel (priv->doc_tree, "info")) {
+		yelp_view_toc_printf (view,
+				      "<a href=\"toc:info\">%s</a><br>\n",
+				      info_string);
+	}
+
+	yelp_view_toc_printf (view,
+			      TOC_BLOCK_END
+			      COLUMN_END 
+			      PAGE_END);
 	
-	yelp_view_toc_write (view, 
-			     "</td>\n", -1);
-
-	yelp_view_toc_write (view, table_end, -1);
-
 	yelp_view_toc_write_footer (view);
 
 	yelp_view_toc_close (view);
@@ -432,11 +521,14 @@ static void
 yelp_view_toc_man_emit (YelpViewTOC *view, GNode *first)
 {
 	YelpViewTOCPriv *priv;
-	GNode *node, *child;
-	YelpSection *section;
-	gboolean got_a_leaf;
-	char *path, *url;
-	int i;
+	GNode           *node, *child;
+	YelpSection     *section;
+	gboolean         got_a_leaf;
+	char            *path, *url;
+	int              i;
+	gboolean         sub_started = FALSE;
+	gchar           *str_docs = _("Documents");
+	gchar           *str_subcats = _("Categories");
 
 	priv = view->priv;
 
@@ -444,22 +536,40 @@ yelp_view_toc_man_emit (YelpViewTOC *view, GNode *first)
 	node = first;
 	do {
 		if (node->children != NULL) {
+			if (!sub_started) {
+				yelp_view_toc_printf (view, TOC_BLOCK_START);
+				yelp_view_toc_printf (view, "<h3>%s</h3>", 
+						      str_subcats);
+				
+				sub_started = TRUE;
+			}
+			
 			child = node->children;
 			
 			section = node->data;
 
 			path = yelp_util_node_to_string_path (node);
-			yelp_view_toc_printf (view, "<h2><a href=\"toc:man/%s\">%s</a></h2>\n", path, section->name);
+			yelp_view_toc_printf (view, "<a href=\"toc:man/%s\">%s</a><br>\n", path, section->name);
 			g_free (path);
 		} else {
 			got_a_leaf = TRUE;
 		}
 	} while ((node = node->next) != NULL);
 
+	if (sub_started) {
+		yelp_view_toc_printf (view, TOC_BLOCK_END);
+		yelp_view_toc_printf (view, TOC_BLOCK_SEPARATOR);
+	}
 
 	if (got_a_leaf) {
+		yelp_view_toc_printf (view,  TOC_BLOCK_START);
+		yelp_view_toc_printf (view, "<h3>%s</h3>",
+				      str_docs);
+
+		yelp_view_toc_write (view, "</td></tr><tr><td>", -1);
+		
 		yelp_view_toc_write (view,
-				     "<table cellpadding=\"2\" cellspacing=\"2\" border=\"0\" width=\"100%%\"><tbody>\n",
+				     "<table cellpadding=\"2\" cellspacing=\"2\" border=\"0\" width=\"100%%\">\n",
 				     -1);
 
 		i = 0;
@@ -481,15 +591,19 @@ yelp_view_toc_man_emit (YelpViewTOC *view, GNode *first)
 				}
 			
 				section = node->data;
-				url = yelp_util_compose_path_url (node->parent, section->uri);
-				yelp_view_toc_printf (view, "<td valign=\"Top\"><a href=\"%s\">%s</a></td>\n", url, section->name);
+				url = yelp_util_compose_path_url (node->parent, yelp_uri_get_path (section->uri));
+				yelp_view_toc_printf (view, "<td valign=\"Top\"><a href=\"%s\">%s</a></td>\n", yelp_uri_to_string (section->uri), 
+						      section->name);
+/* 				yelp_view_toc_printf (view, "<td valign=\"Top\"><a href=\"%s\">%s</a></td>\n", url, section->name); */
 				g_free (url);
 				i++;
 			}
 		} while ((node = node->next) != NULL);
 		
 		yelp_view_toc_write (view, "</tr>\n", -1);
-		yelp_view_toc_write (view, "</tbody></table>\n", -1);
+		yelp_view_toc_write (view, "</table>\n", -1);
+
+		yelp_view_toc_printf (view, TOC_BLOCK_END);
 	}
 }
 
@@ -500,7 +614,7 @@ yelp_view_toc_man_2 (YelpViewTOC *view,
 	GNode *first;
 	gchar *name;
 	gchar *string = _("Manual pages");
-	
+
 	if (root->children == NULL) {
 		return;
 	}
@@ -509,15 +623,42 @@ yelp_view_toc_man_2 (YelpViewTOC *view,
 
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, string);
+	yelp_view_toc_printf (view, PAGE_HEADER, string);
 		
 	name = yelp_view_toc_full_path_name (view, root);
-	
-	yelp_view_toc_printf (view, "<h1><a href=\"toc:man\">%s</a>: '%s'</h1>\n", string, name);
+
+	yelp_view_toc_printf (view, 
+			      "<table cellpadding=\"0\"\n"
+			      "border=\"0\"\n"
+			      "align=\"center\"\n"
+			      "cellspacing=\"10\"\n"
+			      "width=\"100%\">\n"
+			      "<tr align=\"left\"\n"
+			      "    valign=\"top\"\n"
+			      "    bgcolor=\"" BG_COLOR "\">\n"
+			      "<td height=\"69\"\n"
+			      "    align=\"center\"\n"
+			      "    valign=\"center\"\n"
+			      "    bgcolor=\"" BLOCK_BG_COLOR "\"\n"
+			      "    colspan=\"2\">\n"
+			      "  <h2>\n"
+			      "    %s/%s\n"
+			      "  </h2>\n"
+			      "</td>\n"
+			      "</tr>\n"
+			      "<tr>\n",
+			      string, name);
+
 	g_free (name);
 
+	yelp_view_toc_printf (view, COLUMN_RIGHT_START);
+
 	yelp_view_toc_man_emit (view, first);
 		
+	yelp_view_toc_printf (view, 
+			      COLUMN_END
+			      PAGE_END);
+
 	yelp_view_toc_write_footer (view);
 	yelp_view_toc_close (view);
 }
@@ -530,7 +671,8 @@ yelp_view_toc_man_1 (YelpViewTOC *view)
 	YelpSection     *section;
 	char            *path;
 	gchar           *string = _("Manual Pages");
-	
+	gchar           *str_subcats = _("Categories");
+
 	priv = view->priv;
 
 	root = yelp_util_find_toplevel (priv->doc_tree, "man");
@@ -548,23 +690,37 @@ yelp_view_toc_man_1 (YelpViewTOC *view)
 
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, string);
-	
-	yelp_view_toc_printf (view, "<h1>%s</h1>\n", string);
-	
+	yelp_view_toc_printf (view, PAGE_HEADER, string);
+
+	yelp_view_toc_printf (view, 
+			      PAGE_START, 
+			      string);
+
+	yelp_view_toc_printf (view,
+			      COLUMN_RIGHT_START
+			      TOC_BLOCK_START 
+			      "<h2>%s</h2>", 
+			      str_subcats);
+
 	do {
 		child = g_node_first_child (node);
 		
 		if (child) {
 			section = YELP_SECTION (node->data);
 			path = yelp_util_node_to_string_path (node);
-			yelp_view_toc_printf (view, 
-					      "<a href=\"toc:man/%s\">%s</a><br>\n", 
-					      path, section->name);
+
+ 			yelp_view_toc_printf (view,
+ 					      "<a href=\"toc:man/%s\">%s</a><br>\n",
+ 					      path, section->name);
 			g_free (path);
 		}
 	} while ((node = g_node_next_sibling (node)));
-		
+
+	yelp_view_toc_printf (view, 
+			      TOC_BLOCK_END
+			      COLUMN_END
+			      PAGE_END);
+
 	yelp_view_toc_write_footer (view);
 	yelp_view_toc_close (view);
 }
@@ -576,8 +732,9 @@ yelp_view_toc_info (YelpViewTOC *view)
 	GNode           *root, *node;
 	YelpSection     *section;
 	char            *url;
-	gchar           *string = _("Info pages");
-	
+	gchar           *string = _("Info Pages");
+	gchar           *str_docs = _("Documents");
+
 	priv = view->priv;
 
 	root = yelp_util_find_toplevel (priv->doc_tree, "info");
@@ -590,24 +747,40 @@ yelp_view_toc_info (YelpViewTOC *view)
 	node = g_node_first_child (root);
 
 	if (!node) {
+		d(g_print ("No first node\n"));
 		return;
 	}
 
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, _("Info pages"));
-	
-	yelp_view_toc_printf (view, "<h1>%s</h1>\n", string);
-	
+	yelp_view_toc_printf (view, PAGE_HEADER, string);
+
+	yelp_view_toc_printf (view, PAGE_START, string);
+	yelp_view_toc_printf (view, COLUMN_RIGHT_START);
+	yelp_view_toc_printf (view, TOC_BLOCK_START);
+
+	yelp_view_toc_printf (view, "<h2>%s</h2>", str_docs);
+
 	do {
 		section = YELP_SECTION (node->data);
-		url = yelp_util_compose_path_url (root, section->uri);
+		url = yelp_util_compose_path_url (root,
+						  yelp_uri_get_path (section->uri));
+		
 		yelp_view_toc_printf (view, 
 				      "<a href=\"%s\">%s</a><br>\n", 
-				      url, section->name);
+				      yelp_uri_to_string (section->uri),
+				      section->name);
+/* 		yelp_view_toc_printf (view,  */
+/* 				      "<a href=\"%s\">%s</a><br>\n",  */
+/* 				      url, section->name); */
 		g_free  (url);
 	} while ((node = g_node_next_sibling (node)));
 		
+	yelp_view_toc_printf (view, 
+			      TOC_BLOCK_END
+			      COLUMN_END
+			      PAGE_END);
+
 	yelp_view_toc_write_footer (view);
 	yelp_view_toc_close (view);
 }
@@ -615,12 +788,12 @@ yelp_view_toc_info (YelpViewTOC *view)
 static void
 yelp_view_read_important_docs (YelpViewTOC *view)
 {
-	xmlDocPtr doc;
-	xmlNodePtr node;
-	xmlNodePtr child;
-	xmlNodePtr section;
-	xmlNodePtr title;
-	xmlChar *prop;
+	xmlDocPtr   doc;
+	xmlNodePtr  node;
+	xmlNodePtr  child;
+	xmlNodePtr  section;
+	xmlNodePtr  title;
+	xmlChar    *prop;
 	YelpImportantDocsSection *important_section;
 	
 	doc = xmlParseFile (DATADIR "/yelp/important_docs.xml");
@@ -675,6 +848,7 @@ yelp_view_toc_new (GNode *doc_tree)
 {
 	YelpViewTOC     *view;
 	YelpViewTOCPriv *priv;
+	YelpURI         *uri;
 
 	view = g_object_new (YELP_TYPE_VIEW_TOC, NULL);
 
@@ -682,7 +856,10 @@ yelp_view_toc_new (GNode *doc_tree)
 
 	priv->doc_tree = doc_tree;
 
-	yelp_view_toc_open_url (view, "toc:");
+	uri = yelp_uri_new ("toc:");
+	
+	yelp_view_toc_open_uri (view, uri);
+	yelp_uri_unref (uri);
 	
 	return GTK_WIDGET (view);
 }
@@ -697,8 +874,10 @@ yelp_view_toc_scrollkeeper (YelpViewTOC *view,
 	gboolean         got_a_leaf;
 	char            *path;
 	gchar           *name;
-	gchar           *string = _("Scrollkeeper docs for category");
-
+	gboolean         sub_started = FALSE;
+	gchar           *str_docs = _("Documents");
+	gchar           *str_subcats = _("Categories");
+		
 	priv = view->priv;
 	
 	if (root->children == NULL) {
@@ -707,104 +886,127 @@ yelp_view_toc_scrollkeeper (YelpViewTOC *view,
 	
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, "Scrollkeeper");
-		
 	name = yelp_view_toc_full_path_name (view, root);
-	yelp_view_toc_printf (view, "<h1>%s '%s'</h1>\n", string, name);
+
+	yelp_view_toc_printf (view, PAGE_HEADER, name);
+
+	yelp_view_toc_printf (view, PAGE_START, name);
+	
 	g_free (name);
 
+	yelp_view_toc_printf (view, COLUMN_RIGHT_START);
+
 	got_a_leaf = FALSE;
 	node = root->children;
-	while (node != NULL) {
+
+/* 	yelp_view_toc_printf (view, TOC_BLOCK_START); */
+
+	for (node = root->children; node; node = node->next) {
+		
 		if (node->children != NULL) {
+			if (!sub_started) {
+				yelp_view_toc_printf (view, TOC_BLOCK_START);
+				yelp_view_toc_printf (view, "<h2>%s</h2>", 
+						      str_subcats);
+						      
+				sub_started = TRUE;
+			}
+			
 			child = node->children;
 			
 			section = node->data;
 
 			path = yelp_util_node_to_string_path (node);
-			yelp_view_toc_printf (view, "<h2><a href=\"toc:scrollkeeper/%s\">%s</a></h2>\n", path, section->name);
+			yelp_view_toc_printf (view, "<a href=\"toc:scrollkeeper/%s\">%s</a><br>\n", path, section->name);
 			g_free (path);
 		} else {
 			got_a_leaf = TRUE;
 		}
-		node = node->next;
 	}
 
+	if (sub_started) {
+		yelp_view_toc_printf (view, TOC_BLOCK_END);
+	}
+
+/* 	yelp_view_toc_printf (view, TOC_BLOCK_END); */
 
 	if (got_a_leaf) {
-		yelp_view_toc_write (view, "<ul>\n", -1);
+		yelp_view_toc_printf (view, TOC_BLOCK_START);
+		yelp_view_toc_printf (view, "<h2>%s</h2>", str_docs);
+		
+/* 		yelp_view_toc_write (view, "<ul>\n", -1); */
 
-		node = root->children;
-		while (node != NULL) {
+		for (node = root->children; node; node = node->next) {
 			if (node->children == NULL) {
 				YelpSection *section;
 			
 				section = node->data;
-				yelp_view_toc_printf (view, "<li><a href=\"%s\">%s</a>\n", section->uri, section->name);
+				yelp_view_toc_printf (view, "<a href=\"%s\">%s</a><br>\n", yelp_uri_to_string (section->uri), section->name);
 			}
-			node = node->next;
 		}
-		
-		yelp_view_toc_write (view, "</ul>\n", -1);
+
+		yelp_view_toc_printf (view, TOC_BLOCK_END);
 	}
-		
+
+	yelp_view_toc_printf (view, 
+			      COLUMN_END
+			      PAGE_END);
 	yelp_view_toc_write_footer (view);
 	yelp_view_toc_close (view);
 }
 
-
 void
-yelp_view_toc_open_url (YelpViewTOC *view, const char *url)
+yelp_view_toc_open_uri (YelpViewTOC *view, YelpURI *uri)
 {
-	const char *toc_type;
-	const char *path_string;
-	GNode *node;
+	GNode       *node;
+	const gchar *path;
 	
-	g_assert (strncmp (url, "toc:", 4) == 0);
+	g_assert (yelp_uri_get_type (uri) == YELP_URI_TYPE_TOC);
+
+	path = yelp_uri_get_path (uri);
+
+	d(g_print ("PATH:[%s]\n", path));
 	
-	if (strncmp (url, "toc:", 4) != 0) {
-		return;
-	}
-
-	toc_type = url + 4;
-
-	if (*toc_type == 0) {
+	if (!strcmp (path, "")) {
 		yelp_view_toc_start (view);
-	} else if (strncmp (toc_type, "man", 3) == 0) {
-		path_string = toc_type + 3;
-
-		if (path_string[0] == 0) {
+	} 
+	else if (strncmp (path, "man", 3) == 0) {
+		path += 3;
+		if (path[0] == 0) {
  			yelp_view_toc_man_1 (view);
-		} else if (path_string[0] == '/') {
+		} else if (path[0] == '/') {
 			/* Calculate where it should go */
-			path_string++;
+			path++;
 
-			node = yelp_util_string_path_to_node  (path_string,
+			node = yelp_util_string_path_to_node  (path,
 							       view->priv->doc_tree);
 			if (node) {
 				yelp_view_toc_man_2 (view, node);
 			} else {
-				g_warning ("Bad path in toc url %s\n", url);
+				g_warning ("Bad path in toc url %s\n", 
+					   yelp_uri_to_string (uri));
 			}
 		}
-	} else if (strcmp (toc_type, "info") == 0) {
+	} else if (strcmp (path, "info") == 0) {
 		yelp_view_toc_info (view);
-	} else if (strncmp (toc_type, "scrollkeeper", strlen ("scrollkeeper")) == 0) {
-		path_string = toc_type + strlen ("scrollkeeper");
-		if (path_string[0] == '/') {
+	} else if (strncmp (path, "scrollkeeper", strlen ("scrollkeeper")) == 0) {
+		path = path + strlen ("scrollkeeper");
+		if (path[0] == '/') {
 			/* Calculate where it should go */
-			path_string++;
-
-			node = yelp_util_string_path_to_node  (path_string,
+			path++;
+			
+			node = yelp_util_string_path_to_node  (path,
 							       view->priv->doc_tree);
 			if (node) {
 				yelp_view_toc_scrollkeeper (view, node);
 			} else {
-				g_warning ("Bad path in toc url %s\n", url);
+				g_warning ("Bad path in toc url %s\n", 
+					   yelp_uri_to_string (uri));
 			}
 		}
 	} else {
-		g_warning ("Unknown toc type %s\n", url);
+		g_warning ("Unknown toc type %s\n", 
+			   yelp_uri_to_string (uri));
 	}
 
  	gtk_adjustment_set_value (
