@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2001 Mikael Hallendal <micke@codefactory.se>
+ * Copyright (C) 2001-2002 Mikael Hallendal <micke@codefactory.se>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <bonobo/bonobo-main.h>
+#include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnome/gnome-i18n.h>
@@ -48,58 +49,70 @@ typedef enum {
 	YELP_WINDOW_ACTION_FORWARD,
 } YelpHistoryAction;
 
-static void        yw_init		     (YelpWindow          *window);
-static void        yw_class_init	     (YelpWindowClass     *klass);
+static GConfEnumStringPair toolbar_styles[] = {
+        { GTK_TOOLBAR_TEXT, "text" },
+        { GTK_TOOLBAR_ICONS, "icons" },
+        { GTK_TOOLBAR_BOTH, "both" }
+};
 
-static void        yw_populate               (YelpWindow          *window);
+static void        yw_init		       (YelpWindow          *window);
+static void        yw_class_init	       (YelpWindowClass     *klass);
 
-static gboolean    yw_handle_url             (YelpWindow          *window,
-					      const gchar         *url);
-static void        yw_url_selected_cb        (gpointer             view,
-					      char                *url,
-					      char                *base_url,
-					      gboolean             handled,
-					      YelpWindow          *window);
-static void        yw_toggle_history_back    (YelpHistory         *history,
-					      gboolean             sensitive,
-					      YelpWindow          *window);
+static void        yw_populate                 (YelpWindow          *window);
 
-static void        yw_toggle_history_forward (YelpHistory         *history,
-					      gboolean             sensitive,
-					      YelpWindow          *window);
+static gboolean    yw_handle_uri               (YelpWindow          *window,
+						YelpURI             *uri);
+static void        yw_uri_selected_cb          (gpointer             view,
+						YelpURI             *uri,
+						gboolean             handled,
+						YelpWindow          *window);
+static void        yw_toggle_history_back      (YelpHistory         *history,
+						gboolean             sensitive,
+						YelpWindow          *window);
 
-static void        yw_history_action         (YelpWindow          *window,
-					      YelpHistoryAction    action);
-static void        yw_back_button_clicked    (GtkWidget           *button,
-					      YelpWindow          *window);
-static void        yw_forward_button_clicked (GtkWidget           *button,
-					      YelpWindow          *window);
-static void        yw_home_button_clicked    (GtkWidget           *button,
-					      YelpWindow          *window);
-static void        yw_index_button_clicked   (GtkWidget           *button,
-					      YelpWindow          *window);
-static void        yw_new_window_cb          (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static void        yw_close_window_cb        (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static void        yw_exit_cb                (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static void        yw_history_go_cb          (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static void        yw_go_home_cb             (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static void        yw_go_index_cb            (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static void        yw_about_cb               (gpointer             data,
-					      guint                section,
-					      GtkWidget           *widget);
-static GtkWidget * yw_create_toolbar         (YelpWindow          *window);
+static void        yw_toggle_history_forward   (YelpHistory         *history,
+						gboolean             sensitive,
+						YelpWindow          *window);
+
+static void        yw_history_action           (YelpWindow          *window,
+						YelpHistoryAction    action);
+static void        yw_back_button_clicked      (GtkWidget           *button,
+						YelpWindow          *window);
+static void        yw_forward_button_clicked   (GtkWidget           *button,
+						YelpWindow          *window);
+static void        yw_home_button_clicked      (GtkWidget           *button,
+						YelpWindow          *window);
+static void        yw_index_button_clicked     (GtkWidget           *button,
+						YelpWindow          *window);
+static void        yw_new_window_cb            (gpointer             data,
+						guint                section,
+						GtkWidget           *widget);
+static void        yw_close_window_cb          (gpointer             data,
+						guint                section,
+						GtkWidget           *widget);
+static void        yw_history_go_cb            (gpointer             data,
+						guint                section,
+						GtkWidget           *widget);
+static void        yw_go_home_cb               (gpointer             data,
+						guint                section,
+						GtkWidget           *widget);
+static void        yw_go_index_cb              (gpointer             data,
+						guint                section,
+						GtkWidget           *widget);
+static void        yw_about_cb                 (gpointer             data,
+						guint                section,
+						GtkWidget           *widget);
+static GtkWidget * yw_create_toolbar           (YelpWindow          *window);
+
+static void        yw_toolbar_style_changed_cb (GConfClient         *client,
+						guint                cnxn_id,
+						GConfEntry          *entry,
+						gpointer             data);
+
+#if 0
+static void        yw_remove_notify_cb          (GtkObject          *obj, 
+						 gpointer            data);
+#endif
 
 enum {
 	PAGE_TOC_VIEW,
@@ -179,7 +192,8 @@ static void
 yw_init (YelpWindow *window)
 {
         YelpWindowPriv *priv;
-       
+	YelpURI        *uri;
+	
         priv = g_new0 (YelpWindowPriv, 1);
         window->priv = priv;
         
@@ -190,7 +204,9 @@ yw_init (YelpWindow *window)
 	
 	priv->history = yelp_history_new ();
 
-	yelp_history_goto (priv->history, "toc:");
+	uri = yelp_uri_new ("toc:");
+	yelp_history_goto (priv->history, uri);
+	yelp_uri_unref (uri);
 
 	g_signal_connect (priv->history, 
 			  "back_exists_changed",
@@ -299,83 +315,91 @@ yw_populate (YelpWindow *window)
 }
 
 static gboolean
-yw_handle_url (YelpWindow *window, const gchar *url)
+yw_handle_uri (YelpWindow *window, YelpURI *uri)
 {
 	YelpWindowPriv *priv;
-	
+	GError         *error = NULL;
+	gboolean        handled = FALSE;
+
 	priv = window->priv;
 
-	if (strncmp (url, "toc:", 4) == 0) {
-		yelp_view_toc_open_url (YELP_VIEW_TOC (priv->toc_view),
-					url);
+	d(g_print ("Handling URL: %s\n", yelp_uri_to_string (uri)));
+
+/*  	yelp_view_content_stop (YELP_VIEW_CONTENT (priv->content_view)); */
+
+	if (yelp_uri_get_type (uri) == YELP_URI_TYPE_TOC) {
+		d(g_print ("[TOC]\n"));
+		
+		yelp_view_toc_open_uri (YELP_VIEW_TOC (priv->toc_view), uri);
+
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 					       PAGE_TOC_VIEW);
-		return TRUE;
-	} else if (strncmp (url, "man:", 4) == 0 ||
-		   strncmp (url, "info:", 5) == 0 ||
-		   strncmp (url, "ghelp:", 6) == 0 ||
-		   strncmp (url, "path:", 5) == 0) {
-		yelp_view_content_show_uri (YELP_VIEW_CONTENT (priv->content_view),
-					    url);
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
-					       PAGE_CONTENT_VIEW);
-		return TRUE;
-	} else if (strncmp (url, "index:", 6) == 0) {
+		handled = TRUE;
+	} 
+	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_INDEX) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 					       PAGE_INDEX_VIEW);
-		yelp_view_index_show_uri (YELP_VIEW_INDEX (priv->index_view),
-					  url);
-		return TRUE;
+		handled = TRUE;
+	}
+	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_MAN ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_INFO ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_DOCBOOK_XML ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_DOCBOOK_SGML ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_HTML ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_PATH) {
+		d(g_print ("[CONTENT]\n"));
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
+					       PAGE_CONTENT_VIEW);
+		yelp_view_content_show_uri (YELP_VIEW_CONTENT (priv->content_view),
+					    uri,
+					    &error);
+		handled = TRUE;
 	} else {
-		/* FIXME: Show dialog on failure? */
-		gnome_url_show (url, NULL);
+		gnome_url_show (yelp_uri_to_string (uri), &error);
+		handled = FALSE;
 	}
 
-	return FALSE;
+	if (error) {
+		GtkWidget *dialog;
+		
+		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 error->message);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+	}
+
+	return handled;
 }
 
 static void
-yw_url_selected_cb (gpointer    view,
-		    char       *url,
-		    char       *base_url,
+yw_uri_selected_cb (gpointer    view, 
+		    YelpURI    *uri, 
 		    gboolean    handled,
 		    YelpWindow *window)
 {
 	YelpWindowPriv *priv;
-	gchar          *abs_url = NULL;
-
 
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
-	d(g_print ("url_selected: %s base: %s, handled: %d\n", url, base_url, handled));
+	d(g_print ("uri_selected: %s, handled: %d\n", 
+		   yelp_uri_to_string (uri), handled));
 
 	priv = window->priv;
-	
-	if (url && base_url) {
-		abs_url = yelp_util_resolve_relative_uri (base_url, url);
-		
-		d(g_print ("Link '%s' pressed relative to: %s -> %s\n", 
-			   url,
-			   base_url,
-			   abs_url));
-        } else {
-		if (url) {
-			abs_url = g_strdup (url);
-		}
-		else if (base_url) {
-			abs_url = g_strdup (base_url);
-		}
-        }
 
+	yelp_uri_ref (uri);
+	
 	if (handled) {
-		yelp_history_goto (priv->history, abs_url);
+		yelp_history_goto (priv->history, uri);
 	} else {
-		if (yw_handle_url (window, abs_url)) {
-			yelp_history_goto (priv->history, abs_url);
+		if (yw_handle_uri (window, uri)) {
+			yelp_history_goto (priv->history, uri);
 		}
 	}
 
-	g_free (abs_url);
+	yelp_uri_unref (uri);
 }
 
 static void
@@ -422,7 +446,7 @@ static void
 yw_history_action (YelpWindow *window, YelpHistoryAction action)
 {
 	YelpWindowPriv *priv;
-	const gchar    *url;
+	YelpURI        *uri;
 	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -430,17 +454,17 @@ yw_history_action (YelpWindow *window, YelpHistoryAction action)
 
 	switch (action) {
 	case YELP_WINDOW_ACTION_BACK:
-		url = yelp_history_go_back (priv->history);
+		uri = yelp_history_go_back (priv->history);
 		break;
 	case YELP_WINDOW_ACTION_FORWARD:
-		url = yelp_history_go_forward (priv->history);
+		uri = yelp_history_go_forward (priv->history);
 		break;
 	default:
 		return;
 	}
 	
-	if (url) {
-		yw_handle_url (window, url);
+	if (uri) {
+		yw_handle_uri (window, uri);
 	}
 }
 
@@ -465,12 +489,16 @@ yw_forward_button_clicked (GtkWidget *button, YelpWindow *window)
 static void
 yw_home_button_clicked (GtkWidget *button, YelpWindow *window)
 {
-	g_return_if_fail (YELP_IS_WINDOW (window));
-
-	yelp_history_goto (window->priv->history, "toc:");
+	YelpURI *uri;
 	
-	yelp_view_toc_open_url (YELP_VIEW_TOC (window->priv->toc_view),
-				"toc:");
+	g_return_if_fail (YELP_IS_WINDOW (window));
+	
+	uri = yelp_uri_new ("toc:");
+
+	yelp_history_goto (window->priv->history, uri);
+	yelp_view_toc_open_uri (YELP_VIEW_TOC (window->priv->toc_view), uri);
+
+	yelp_uri_unref (uri);
 
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
 				       PAGE_TOC_VIEW);
@@ -479,8 +507,14 @@ yw_home_button_clicked (GtkWidget *button, YelpWindow *window)
 static void
 yw_index_button_clicked (GtkWidget *button, YelpWindow *window)
 {
+	YelpURI *uri;
+	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
+	uri = yelp_uri_new ("index:");
+	yelp_history_goto (window->priv->history, uri);
+	yelp_uri_unref (uri);
+	
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
 				       PAGE_INDEX_VIEW);
 }
@@ -501,12 +535,6 @@ yw_close_window_cb (gpointer   data,
 	gtk_widget_destroy (GTK_WIDGET (data));
 }
 
-static void
-yw_exit_cb (gpointer data, guint section, GtkWidget *widget)
-{
-	bonobo_main_quit ();
-}
-
 static void
 yw_history_go_cb (gpointer data, guint section, GtkWidget *widget)
 {
@@ -552,17 +580,44 @@ yw_about_cb (gpointer data, guint section, GtkWidget *widget)
 static GtkWidget *
 yw_create_toolbar (YelpWindow *window)
 {
-	YelpWindowPriv *priv;
-	GtkWidget      *toolbar;
-	GtkWidget      *button;
-	GtkWidget      *icon;
-
+	YelpWindowPriv  *priv;
+	GtkWidget       *toolbar;
+	GtkWidget       *button;
+	GtkWidget       *icon;
+	GConfClient     *conf_client;
+	gchar           *str;
+	GtkToolbarStyle  style = GTK_TOOLBAR_BOTH;
+	
 	g_return_val_if_fail (YELP_IS_WINDOW (window), NULL);
 
 	priv = window->priv;
 
 	toolbar = gtk_toolbar_new ();
 
+	conf_client = gconf_client_get_default ();
+	
+	str = gconf_client_get_string (conf_client, 
+				       "/desktop/gnome/interface/toolbar_style",
+				       NULL);
+
+	if (str) {
+		gconf_string_to_enum (toolbar_styles,
+				      str,
+				      (gint*)&style);
+		g_free (str);
+	}
+
+	gconf_client_notify_add (conf_client, 
+				 "/desktop/gnome/interface/toolbar_style",
+				 yw_toolbar_style_changed_cb,
+				 toolbar, NULL, NULL);
+	
+/* 	g_signal_connect(toolbar, "destroy", */
+/* 			 G_CALLBACK(yw_remove_notify_cb), */
+/* 			 GINT_TO_POINTER(notify_id)); */
+
+	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), style);
+
 	icon = gtk_image_new_from_stock ("gtk-go-back", 
 					 GTK_ICON_SIZE_LARGE_TOOLBAR);
 
@@ -615,6 +670,44 @@ yw_create_toolbar (YelpWindow *window)
 	return toolbar;
 }
 
+static void
+yw_toolbar_style_changed_cb (GConfClient *client,
+			     guint        cnxn_id,
+			     GConfEntry  *entry,
+			     gpointer     data)
+{
+        GtkToolbarStyle  style   = GTK_TOOLBAR_BOTH;
+        GtkToolbar      *toolbar = GTK_TOOLBAR(data);
+	GConfValue      *value;
+
+	value = gconf_entry_get_value (entry);
+
+        /* If no per-app setting use this new global setting */
+        if (value && 
+	    value->type == GCONF_VALUE_STRING &&
+            gconf_value_get_string (value) != NULL) {
+                gconf_string_to_enum(toolbar_styles,
+                                     gconf_value_get_string(value),
+                                     (gint*)&style);
+        }
+
+        gtk_toolbar_set_style(toolbar, style);
+}
+
+#if 0
+static void
+yw_remove_notify_cb (GtkObject *obj, gpointer data)
+{
+        GConfClient *conf_client;
+        guint        notify_id;
+
+        conf_client = gconf_client_get_default ();
+        notify_id   = GPOINTER_TO_INT(data);
+
+        gconf_client_notify_remove(conf_client, notify_id);
+}
+#endif 
+
 GtkWidget *
 yelp_window_new (GNode *doc_tree, GList *index)
 {
@@ -627,22 +720,22 @@ yelp_window_new (GNode *doc_tree, GList *index)
 	priv->doc_tree = doc_tree;
 	priv->index    = index;
 
- 	priv->toc_view    = yelp_view_toc_new (doc_tree);
+ 	priv->toc_view     = yelp_view_toc_new (doc_tree);
  	priv->content_view = yelp_view_content_new (doc_tree);
 
 	if (priv->index) {
-		priv->index_view   = yelp_view_index_new (index);
+		priv->index_view = yelp_view_index_new (index);
 
-		g_signal_connect (priv->index_view, "url_selected",
-				  G_CALLBACK (yw_url_selected_cb),
+		g_signal_connect (priv->index_view, "uri_selected",
+				  G_CALLBACK (yw_uri_selected_cb),
 				  window);
 	} 
 
-	g_signal_connect (priv->toc_view, "url_selected",
-			  G_CALLBACK (yw_url_selected_cb),
+	g_signal_connect (priv->toc_view, "uri_selected",
+			  G_CALLBACK (yw_uri_selected_cb),
 			  window);
-	g_signal_connect (priv->content_view, "url_selected",
-			  G_CALLBACK (yw_url_selected_cb),
+	g_signal_connect (priv->content_view, "uri_selected",
+			  G_CALLBACK (yw_uri_selected_cb),
 			  window);
 
 	yw_populate (window);
@@ -655,18 +748,26 @@ yelp_window_open_uri (YelpWindow  *window,
 		      const gchar *str_uri)
 {
 	YelpWindowPriv *priv;
+	YelpURI        *uri;
 	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 	
 	priv = window->priv;
 
-	yw_handle_url (window, str_uri);
+	uri = yelp_uri_new (str_uri);
+
+	if (yelp_uri_exists (uri)) {
+		yw_handle_uri (window, uri);
+	}
+	
+	yelp_uri_unref (uri);
 }
 
-const gchar *
+YelpURI *
 yelp_window_get_current_uri (YelpWindow *window)
 {
 	g_return_val_if_fail (YELP_IS_WINDOW (window), NULL);
 	
 	return yelp_history_get_current (window->priv->history);
 }
+
